@@ -28,6 +28,9 @@
 /* Define if your system has XkbBell().  */
 #undef HAVE_XKB_BELL_EXT
 
+/* Define if your system has XInput version 2.1 or higher.  */
+#undef HAVE_XI21
+
 /* Define if your system allows setsid() within vfork().  */
 #undef HAVE_GOOD_SETSID_VFORK
 
