@@ -1693,6 +1693,10 @@ geom_do_char(g_info, ch)
 	    }
 	    maxchar = currinf.fontp->maxchar;
 	    currinf.set_char_p = currinf.fontp->set_char_p;
+#if FREETYPE
+	    if (currinf.set_char_p == set_ft_char)
+		do_load_freetype_font();
+#endif
 	}
 
 	if (currinf.set_char_p == set_char) {
