@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2004  Paul Vojta
+Copyright (c) 1990-2013  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -31,6 +31,11 @@ NOTE:
 #include "dvi.h"
 #include <sys/stat.h>
 
+#if FREETYPE
+# include <ft2build.h>
+# include FT_SIZES_H
+#endif
+
 #if !defined(S_ISDIR) && defined(S_IFDIR)
 #define	S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
 #endif
@@ -248,9 +253,7 @@ load_font(fontp)
 	int	magic;
 	Boolean	hushcs	= hush_chk;
 
-	fontp->file = font_open(fontp->fontname, &font_found,
-	    fsize, &size_found, fontp->magstepval, &fontp->filename);
-	if (fontp->file == NULL) {
+	if (!font_open(fontp, &font_found, &size_found)) {
 	    if (ev_flags & EV_GE_NEWDOC)
 		return False;
 	    fontp->flags |= FONT_LOADED;	/* as loaded as it'll get */
@@ -271,10 +274,18 @@ load_font(fontp)
 	    Fprintf(stderr,
 		"%s: can't find font %s at %d dpi; using %d dpi instead.\n",
 		prog, fontp->fontname, dpi, size_found);
-	fontp->fsize = size_found;
 	fontp->timestamp = ++current_timestamp;
 	fontp->maxchar = maxchar = 255;
 	fontp->set_char_p = set_char;
+#if FREETYPE
+	if (fontp->ft != NULL) {	/* if freetype font */
+	    fontp->set_char_p = set_ft_char;
+	    fontp->glyph = xmalloc(256 * sizeof(struct glyph));
+	    bzero((char *) fontp->glyph, 256 * sizeof(struct glyph));
+	    return True;
+	}
+#endif
+	fontp->fsize = size_found;
 	magic = two(fontp->file);
 	if (magic == PK_MAGIC) read_PK_index(fontp, WIDENINT hushcs);
 	else
@@ -434,11 +445,20 @@ define_font(file, cmnd, vfparent, tntable, tn_table_len, tn_headpp)
 		fontp->fontname = fontname;
 		fontp->fsize = fsize;
 		fontp->magstepval = magstepval;
-		fontp->file = NULL;	/* needed if it's a virtual font */
+		fontp->file = NULL;	/* needed for virtual/freetype fonts */
+		fontp->filename = NULL;	/* needed for freetype fonts */
 		fontp->checksum = checksum;
 		fontp->flags = FONT_IN_USE;
 		fontp->dimconv = scale * scale_dimconv / (1<<20);
 		fontp->set_char_p = load_n_set_char;
+#if FREETYPE
+		fontp->ft = NULL;
+		/* spsize = scaled size of the font in spell units,
+		 *	  = scale * [vfparent->]dimconv.
+		 */
+		fontp->spsize =
+		  (vfparent != NULL ? vfparent->dimconv : dimconv) * scale;
+#endif
 		if (vfparent == NULL)
 		    if (!load_font(fontp)) {
 			if (ev_flags & EV_GE_NEWDOC) {	/* if aborting */
@@ -591,11 +611,50 @@ read_postamble()
 		*fontpp = fontp->next;		/* remove from list */
 		free(fontp->fontname);
 		if (fontp->flags & FONT_LOADED) {
+#if FREETYPE
+		    if (fontp->ft != NULL) {	/* if FreeType font */
+			struct ftfont	*ft;
+
+			ft = fontp->ft;
+			if (fontp->size != NULL)
+			    FT_Done_Size(fontp->size);
+			if (fontp == ft->first_size) {
+			    if (fontp->next_size == NULL) {
+				/* if this is the only size of this font face */
+				FT_Done_Face(ft->face);
+				ft->t1->ft = NULL;
+				free(ft);
+			    }
+			    else {
+				struct font	*fp2;
+
+				ft->first_size = fp2 = fontp->next_size;
+				fp2->file = fontp->file;
+				fontp->file = NULL;
+				fp2->filename = fontp->filename;
+				fontp->filename = NULL;
+				fp2->timestamp = fontp->timestamp;
+			    }
+			}
+			else {
+			    struct font *fp2;
+
+			    fp2 = ft->first_size;
+			    while (fp2->next_size != fontp)
+				fp2 = fp2->next_size;
+			    fp2->next_size = fontp->next_size;
+			}
+		    }
+#endif
 		    if (fontp->file != NULL) {
 			Fclose(fontp->file);
 			++n_files_left;
 		    }
-		    free(fontp->filename);
+#if FREETYPE
+		    if (fontp->filename != NULL)
+#endif
+			free((void *) fontp->filename);
+
 		    if (fontp->flags & FONT_VIRTUAL) {
 			struct macro *m;
 
@@ -701,7 +760,7 @@ form_dvi_property(inode)
 
 	/* Do it this way because different copies of xdvi might be running on
 	   machines with different endiannesses.  */
-	dvi_inode = ino;
+	ino = dvi_inode = inode;
 	for (i = 0; i < 8; ++i) {
 	    dvi_property[i] = ino;
 	    ino >>= 8;
@@ -886,6 +945,7 @@ check_dvi_file()
 	    ev_flags |= EV_NEWDOC;
 	    Fclose(dvi_file);
 	    dvi_file = NULL;
+	    dvi_file_ready = False;
 	    return False;
 	}
 
@@ -901,7 +961,7 @@ reload_dvi_file()
 {
 	struct font	*fontp;
 
-	dvi_is_valid = False;
+	dvi_is_valid = dvi_file_ready = False;
 	if (page_info != NULL) {
 	    free((char *) (page_info - 1));
 	    page_info = NULL;
