@@ -739,6 +739,7 @@ gs_path_fopen(name, proc)
 	_Xconst	char	*str1_end, *str2_end;
 	_Xconst char	*p1, *p2;
 	FILE		*f;
+	unsigned int	namelen;
 
 	if (str1 == NULL) {
 	    str1 = str2;
@@ -787,6 +788,11 @@ gs_path_fopen(name, proc)
 	    str1 = p1 + 1;
 	}
 
+	/* leave the file name in ffline[] for error message */
+	namelen = strlen(name) + 1;
+	if (namelen > ffline_len) expandline(namelen);
+	memcpy(ffline, name, namelen);
+
 	return NULL;
 }
 
@@ -1112,6 +1118,7 @@ process_gs_fontmap P1C(FILE *, f)
 	 * Allow entries of the following types:
 	 *
 	 *	(string) .runlibfile
+	 *	(string) .runlibfileifexists
 	 *	/identifier (string) ;
 	 *	/identifier /alias ;
 	 */
@@ -1121,6 +1128,7 @@ process_gs_fontmap P1C(FILE *, f)
 	    if (ttype == GS_EOF || ttype == GS_ERR)
 		break;
 	    if (ttype == LPAREN) {
+		Boolean	quiet = False;
 		FILE	*f1;
 
 		ttype = get_gs_token(&gsf, pos1, &pos2, "Fontmap");
@@ -1131,7 +1139,10 @@ process_gs_fontmap P1C(FILE *, f)
 		      "%s: unexpected end of Fontmap file; giving up.\n", prog);
 		    break;
 		}
-		if (ttype != '.' || pos2 - pos1 != 11
+		if (ttype == '.' && pos2 - pos1 == 19
+		  && memcmp(ffline + pos1, ".runlibfileifexists", 19) == 0)
+		    quiet = True;
+		else if (ttype != '.' || pos2 - pos1 != 11
 		  || memcmp(ffline + pos1, ".runlibfile", 11) != 0) {
 		    Fprintf(stderr, "%s: invalid token following \"(%.*s)\" in Fontmap file; giving up.\n",
 		      prog, (int) pos1, ffline);
@@ -1150,9 +1161,14 @@ process_gs_fontmap P1C(FILE *, f)
 		    free(q);
 		}
 
-		if (f1 == NULL)
-		    Fprintf(stderr, "%s: Fontmap .runlibfile: %s: %s\n",
-		      prog, ffline, strerror(errno));
+		if (f1 == NULL) {
+		    if (!quiet)
+			Fprintf(stderr, "%s: Fontmap .runlibfile: %s: %s\n",
+			  prog, ffline, strerror(errno));
+		    else if (debug & DBG_OPEN)
+			Printf("Fontmap .runlibfileifexists: %s: %s\n",
+			  ffline, strerror(errno));
+		}
 		else {
 		    --gs_fontmap_number;
 		    process_gs_fontmap(f1);
@@ -1411,8 +1427,8 @@ open_t1_font(t1p, path_ret)
 	    f = lookup_gs_font(t1p->psname, path_ret);
 
 	    if (f == NULL) {
-		Fprintf(stderr, "%s: cannot find Type 1 font %s\n", prog,
-		  t1p->psname);
+		if (debug & DBG_OPEN)
+		    Printf("Cannot find Type 1 font %s\n", t1p->psname);
 		return NULL;
 	    }
 
@@ -1423,8 +1439,9 @@ open_t1_font(t1p, path_ret)
 	    f = filefind(t1p->fontfile, &search_type1, path_ret);
 
 	    if (f == NULL) {
-		Fprintf(stderr, "%s: cannot find Type 1 font file %s (will try PK version instead)\n",
-		  prog, t1p->fontfile);
+		if (debug & DBG_OPEN)
+		    Printf("Cannot find Type 1 font file %s (will try PK version instead)\n",
+		      t1p->fontfile);
 		return NULL;
 	    }
 	}
