@@ -859,10 +859,6 @@ static	int		nticks		= 0;	/* number of records total */
 #define	TICKCACHESIZE	3
 #endif
 
-#ifndef	TICKTMP
-#define	TICKTMP		"/tmp"
-#endif
-
 /*
  *	cachetick() - returns:
  *		NULL		error;
