@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2004  Paul Vojta
+Copyright (c) 1990-2013  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -40,14 +40,6 @@ NOTE:
 # include <sys/stat.h>
 #endif
 
-#ifndef WORD64
-# ifdef LONG64
-typedef	unsigned int	xuint32;
-# else
-typedef	unsigned long	xuint32;
-# endif
-#endif
-
 #ifdef VMS
 #include <rmsdef.h>
 #endif /* VMS */
@@ -118,44 +110,35 @@ extern	void	*realloc();
 static	void
 exit_clean_windows()
 {
-	unsigned char	*window_list;
-	size_t		window_list_len;
-	unsigned char	*window_list_end;
-	unsigned char	*wp;
-
-#ifndef WORD64
-# define W(p)	(*((xuint32 *) p))
-#else
-# if WORDS_BIGENDIAN
-#  define W(p)	(((unsigned long) p[0] << 24) | ((unsigned long) p[1] << 16) \
-		  | ((unsigned long) p[2] << 8) | (unsigned long) p[3])
-# else
-#  define W(p)	(((unsigned long) p[3] << 24) | ((unsigned long) p[2] << 16) \
-		  | ((unsigned long) p[1] << 8) | (unsigned long) p[0])
-# endif
-#endif
+	long	*window_list;
+	size_t	window_list_len;
+	long	*window_list_end;
+	long	*wp;
 
 	window_list_len = property_get_data(DefaultRootWindow(DISP),
-	  ATOM_XDVI_WINDOWS, &window_list, XGetWindowProperty);
+	  ATOM_XDVI_WINDOWS, (unsigned char **) &window_list,
+	  XGetWindowProperty);
 
 	if (window_list_len == 0)
 	    return;
 
-	if (window_list_len % 4 != 0) {
+	if (window_list_len % sizeof(long) != 0) {
 	    XDeleteProperty(DISP, DefaultRootWindow(DISP), ATOM_XDVI_WINDOWS);
 	    return;
 	}
 
+	window_list_len /= sizeof(long);
+
 	/* Loop over list of windows.  */
 
 	window_list_end = window_list + window_list_len;
 
-	for (wp = window_list; wp < window_list_end; wp += 4)
-	    if (W(wp) == XtWindow(top_level)) {
+	for (wp = window_list; wp < window_list_end; ++wp)
+	    if (*wp == XtWindow(top_level)) {
 
-		window_list_len -= 4;
-		window_list_end -= 4;
-		bcopy(wp + 4, wp, window_list_end - wp);
+		--window_list_len;
+		--window_list_end;
+		memcpy(wp, wp + 1, (window_list_end - wp) * sizeof(long));
 
 		if (window_list_len == 0)
 		    XDeleteProperty(DISP, DefaultRootWindow(DISP),
@@ -163,7 +146,7 @@ exit_clean_windows()
 		else
 		    XChangeProperty(DISP, DefaultRootWindow(DISP),
 		      ATOM_XDVI_WINDOWS, ATOM_XDVI_WINDOWS, 32, PropModeReplace,
-		      (unsigned char *) window_list, window_list_len / 4);
+		      (unsigned char *) window_list, window_list_len);
 
 		XFlush(DISP);
 		return;
@@ -175,7 +158,6 @@ exit_clean_windows()
 
 	return;
 }
-#undef	W
 
 
 /*
@@ -712,6 +694,160 @@ snum(fp, size)
 	return x;
 }
 
+
+#if FREETYPE || PS
+
+/*
+ *	General AVL tree mechanism.  Search for a node, and return it if found.
+ *	Otherwise insert a node.
+ *	This uses the AVL algorithm from Knuth Vol. 3.
+ */
+
+struct avl *
+avladd(key, key_len, headp, size)
+	_Xconst char	*key;
+	size_t		key_len;
+	struct avl	**headp;
+	size_t		size;
+{
+	struct avl	*ap;
+	struct avl	**app;
+	struct avl	*sp;	/* place where rebalancing may be necessary */
+	struct avl	**spp;	/* points to sp */
+	int		i;
+
+	/* Search */
+	spp = app = headp;
+	for (;;) {
+	    ap = *app;
+	    if (ap == NULL)	/* bottom of tree */
+		break;
+	    if (ap->bal != 0)
+		spp = app;
+	    i = key_len - ap->key_len;
+	    if (i == 0)
+		i = memcmp(key, ap->key, key_len);
+	    if (i == 0)		/* found record already */
+		return ap;
+	    if (i < 0)		/* move left */
+		app = &ap->left;
+	    else
+		app = &ap->right;
+	}
+
+	/* Insert */
+	ap = xmalloc(size);
+	ap->key = key;
+	ap->key_len = key_len;
+	ap->bal = 0;
+	ap->left = ap->right = NULL;
+	*app = ap;
+
+	/* Adjust balance factors */
+	sp = *spp;
+	if (sp == ap)
+	    return ap;
+	i = key_len - sp->key_len;
+	if (i == 0)
+	    i = memcmp(key, sp->key, key_len);
+	sp = (i < 0 ? sp->left : sp->right);
+	while (sp != ap) {
+	    i = key_len - sp->key_len;
+	    if (i == 0)
+		i = memcmp(key, sp->key, key_len);
+	    if (i < 0) {
+		sp->bal = -1;
+		sp = sp->left;
+	    }
+	    else {
+		sp->bal = 1;
+		sp = sp->right;
+	    }
+	}
+
+	/* Balancing act */
+	sp = *spp;
+	i = key_len - sp->key_len;
+	if (i == 0)
+	    i = memcmp(key, sp->key, key_len);
+	if (i < 0) {
+	    if (sp->bal >= 0)
+		--sp->bal;
+	    else {	/* need to rebalance */
+		struct avl *left;
+
+		left = sp->left;
+		if (left->bal < 0) {	/* single rotation */
+		    sp->left = left->right;
+		    left->right = sp;
+		    sp->bal = left->bal = 0;
+		    *spp = left;
+		}
+		else {			/* double rotation */
+		    struct avl	*newtop;
+
+		    newtop = left->right;
+		    sp->left = newtop->right;
+		    newtop->right = sp;
+		    left->right = newtop->left;
+		    newtop->left = left;
+		    sp->bal = left->bal = 0;
+		    if (newtop->bal < 0) ++sp->bal;
+		    else if (newtop->bal > 0) --left->bal;
+		    newtop->bal = 0;
+		    *spp = newtop;
+		}
+	    }
+	}
+	else {
+	    if (sp->bal <= 0)
+		++sp->bal;
+	    else {	/* need to rebalance */
+		struct avl *right;
+
+		right = sp->right;
+		if (right->bal > 0) {	/* single rotation */
+		    sp->right = right->left;
+		    right->left = sp;
+		    sp->bal = right->bal = 0;
+		    *spp = right;
+		}
+		else {			/* double rotation */
+		    struct avl	*newtop;
+
+		    newtop = right->left;
+		    sp->right = newtop->left;
+		    newtop->left = sp;
+		    right->left = newtop->right;
+		    newtop->right = right;
+		    sp->bal = right->bal = 0;
+		    if (newtop->bal > 0) --sp->bal;
+		    else if (newtop->bal < 0) ++right->bal;
+		    newtop->bal = 0;
+		    *spp = newtop;
+		}
+	    }
+	}
+
+	return ap;
+}
+
+#endif /* FREETYPE || PS */
+
+
+/*
+ *	On 64-bit platforms, XGetWindowProperty and related functions convert
+ *	properties with format=32 to arrays of longs.  This function keeps that
+ *	convention.
+ *	The return value is the total number of bytes in the buffer.
+ */
+
+#if defined(WORD64) || defined(LONG64)
+# define LONG_CONV_64(bytes, format)	((bytes) << ((format) >> 5))
+#else
+# define LONG_CONV_64(bytes, format)	(bytes)
+#endif
+
 size_t
 property_get_data(w, a, ret_buf, x_get_property)
 	Window		w;
@@ -745,14 +881,18 @@ property_get_data(w, a, ret_buf, x_get_property)
 
 	    nitems_ret *= (format_ret / 8);	/* convert to bytes */
 
-	    while ((byte_offset + nitems_ret) >= buffer_len) {
-		buffer_len += 256;
+	    if (LONG_CONV_64(byte_offset + nitems_ret, format_ret)
+	      >= buffer_len) {
+		buffer_len += 256
+		  * ((LONG_CONV_64(byte_offset + nitems_ret, format_ret)
+		  - buffer_len) / 256 + 1);
 		buffer = (buffer == NULL ? xmalloc(buffer_len)
 		  : xrealloc(buffer, buffer_len));
 	    }
 
 	    /* the +1 captures the extra '\0' that Xlib puts after the end.  */
-	    memcpy(buffer + byte_offset, prop_ret, nitems_ret + 1);
+	    memcpy(buffer + LONG_CONV_64(byte_offset, format_ret), prop_ret,
+	      LONG_CONV_64(nitems_ret, format_ret) + 1);
 	    byte_offset += nitems_ret;
 
 	    XFree(prop_ret);
@@ -766,7 +906,7 @@ property_get_data(w, a, ret_buf, x_get_property)
 	    XFree(prop_ret);
 
 	*ret_buf = buffer;
-	return byte_offset;
+	return LONG_CONV_64(byte_offset, format_ret);
 }
 
 
