@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2013  Paul Vojta
+Copyright (c) 1990-2014  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -89,6 +89,10 @@ NOTE:
 #include <X11/Xutil.h>	/* needed for XDestroyImage */
 #include <X11/Xos.h>
 
+#if HAVE_XI21
+# include <X11/extensions/XInput2.h>	/* for hi-res (smooth) scrolling */
+#endif
+
 #if	XlibSpecificationRelease >= 5
 #include <X11/Xfuncs.h>
 #endif
@@ -627,6 +631,9 @@ extern	struct _resource {
 	Boolean		_list_fonts;
 #if FREETYPE
 	Boolean		freetype;
+#endif
+#if HAVE_XI21
+	Boolean		xi2scrolling;
 #endif
 	Boolean		reverse;
 	Boolean		_warn_spec;
@@ -879,13 +886,55 @@ struct wheel_acts {
 	struct xdvi_action *action;
 };
 
-EXTERN	struct wheel_acts	*wheel_actions;
+EXTERN	struct wheel_acts	*wheel_actions	INIT(NULL);
 #endif
 
-#if MOTIF && BUTTONS
+#if BUTTONS
 EXTERN	XtTranslations	wheel_trans_table	INIT(NULL);
 #endif
 
+#if HAVE_XI21
+struct xi2_valinfo {
+	int		number;
+	double		increment;
+	double		lastval;
+# if TOOLKIT
+	double		lastexact;
+	double		factor;
+# endif
+	unsigned long	serial;
+};
+
+struct xi2_slave {
+	int		id;			/* slave device id */
+	unsigned int	flags;
+	unsigned char	enabled;
+	unsigned char	btn_mask;	/* buttons corresponding to valuators */
+	struct xi2_valinfo vert, horiz;
+	struct xi2_slave *next;
+};
+
+	/* flag bits */
+#define	XI2_SLAVE_VERT			(1<<0)	/* if vert. valuator present */
+#define	XI2_SLAVE_HORIZ			(1<<1)	/* if horiz. valuator present */
+
+struct xi2_master {
+	int		id;			/* master device id */
+	struct xi2_slave *slave;		/* currently active slave dev */
+	struct xi2_master *next;
+};
+
+
+EXTERN	int		xi2_opcode;
+EXTERN	Boolean		xi2_active		INIT(False);
+EXTERN	struct xi2_master *xi2_masters;		/* linked list of master devs */
+EXTERN	struct xi2_master *xi2_current;		/* current master device */
+EXTERN	struct xi2_slave *xi2_slaves;		/* linked list of slave devs */
+
+EXTERN	struct xi2_slave xi2_no_slave;		/* if no slave assigned yet */
+
+#endif /* HAVE_XI21 */
+
 #ifdef	GREY
 EXTERN	Pixel		*pixeltbl;
 EXTERN	Pixel		*pixeltbl_t;
@@ -987,8 +1036,8 @@ EXTERN	Widget		x_bar, y_bar;	/* horizontal and vert. scroll bars */
 EXTERN	Widget		form_widget;
 EXTERN	int		xtra_wid	INIT(0);
 extern	_Xconst char	default_button_config[]; /* defined in events.c */
-#  if XAW
 EXTERN	Widget		panel_widget;
+#  if XAW
 EXTERN	Cursor		panel_cursor	INIT(0);
 #  endif
 # endif
@@ -1118,12 +1167,6 @@ struct geom_info {
 	void	*geom_data;
 };
 
-typedef	void	(*mouse_proc)	ARGS((XEvent *));
-extern	void	null_mouse	ARGS((XEvent *));
-
-EXTERN	mouse_proc	mouse_motion	INIT(null_mouse);
-EXTERN	mouse_proc	mouse_release	INIT(null_mouse);
-
 /* Used for source special lookup (forward search) and for window manager
    delete protocol (toolkit only).  */
 
@@ -1174,6 +1217,14 @@ extern	struct psprocs	{
 _XFUNCPROTOBEGIN
 
 extern	int	atopix ARGS((_Xconst char *, wide_bool));
+#if HAVE_XI21
+extern	void	xi2_init_valuators ARGS((struct xi2_slave *,
+		  XIAnyClassInfo **, int));
+# if TOOLKIT
+extern	void	xi2_activate_widget ARGS((Widget));
+# endif
+extern	void	xi2_activate ARGS((void));
+#endif /* HAVE_XI21 */
 #if TOOLKIT
 extern	Bool	compile_action ARGS((_Xconst char *, struct xdvi_action **));
 #endif
